[![Git](https://git-scm.com/images/logo@2x.png)](https://git-scm.com/) --distributed-even-if-your-workflow-isnt
![](https://git-scm.com/images/dark-mode.svg)
  * [About](https://git-scm.com/about)
    * [Trademark](https://git-scm.com/about/trademark)
  * [Learn](https://git-scm.com/learn)
    * [Book](https://git-scm.com/book)
    * [Cheat Sheet](https://git-scm.com/cheat-sheet)
    * [Videos](https://git-scm.com/videos)
    * [External Links](https://git-scm.com/doc/ext)
  * [Tools](https://git-scm.com/tools)
    * [Command Line](https://git-scm.com/tools/command-line)
    * [GUIs](https://git-scm.com/tools/guis)
    * [Hosting](https://git-scm.com/tools/hosting)
  * [Reference](https://git-scm.com/docs)
  * [Install](https://git-scm.com/install/linux)
  * [Community](https://git-scm.com/community)


  * Table of Contents
    * [NAME](https://git-scm.com/docs/gitremote-helpers#_name)
    * [SYNOPSIS](https://git-scm.com/docs/gitremote-helpers#_synopsis)
    * [DESCRIPTION](https://git-scm.com/docs/gitremote-helpers#_description)
    * [INVOCATION](https://git-scm.com/docs/gitremote-helpers#_invocation)
    * [INPUT FORMAT](https://git-scm.com/docs/gitremote-helpers#_input_format)
    * [COMMANDS](https://git-scm.com/docs/gitremote-helpers#_commands)
    * [REF LIST ATTRIBUTES](https://git-scm.com/docs/gitremote-helpers#_ref_list_attributes)
    * [REF LIST KEYWORDS](https://git-scm.com/docs/gitremote-helpers#_ref_list_keywords)
    * [OPTIONS](https://git-scm.com/docs/gitremote-helpers#_options)
    * [SEE ALSO](https://git-scm.com/docs/gitremote-helpers#_see_also)
    * [GIT](https://git-scm.com/docs/gitremote-helpers#_git)


[ English ▾](https://git-scm.com/docs/gitremote-helpers)
Localized versions of **gitremote-helpers** manual
  1. [English ](https://git-scm.com/docs/gitremote-helpers)

Want to read in your language or fix typos?
[You can help translate this page](https://github.com/jnavila/git-manpages-l10n).
[Topics ▾](https://git-scm.com/docs/gitremote-helpers)
### Setup and Config
  * [ git ](https://git-scm.com/docs/git)
  * [ config ](https://git-scm.com/docs/git-config)
  * [ help ](https://git-scm.com/docs/git-help)
  * [ bugreport ](https://git-scm.com/docs/git-bugreport)
  * [ Credential helpers ](https://git-scm.com/doc/credential-helpers)


### Getting and Creating Projects
  * [ init ](https://git-scm.com/docs/git-init)
  * [ clone ](https://git-scm.com/docs/git-clone)


### Basic Snapshotting
  * [ add ](https://git-scm.com/docs/git-add)
  * [ status ](https://git-scm.com/docs/git-status)
  * [ diff ](https://git-scm.com/docs/git-diff)
  * [ commit ](https://git-scm.com/docs/git-commit)
  * [ notes ](https://git-scm.com/docs/git-notes)
  * [ restore ](https://git-scm.com/docs/git-restore)
  * [ reset ](https://git-scm.com/docs/git-reset)
  * [ rm ](https://git-scm.com/docs/git-rm)
  * [ mv ](https://git-scm.com/docs/git-mv)


### Branching and Merging
  * [ branch ](https://git-scm.com/docs/git-branch)
  * [ checkout ](https://git-scm.com/docs/git-checkout)
  * [ switch ](https://git-scm.com/docs/git-switch)
  * [ merge ](https://git-scm.com/docs/git-merge)
  * [ mergetool ](https://git-scm.com/docs/git-mergetool)
  * [ log ](https://git-scm.com/docs/git-log)
  * [ stash ](https://git-scm.com/docs/git-stash)
  * [ tag ](https://git-scm.com/docs/git-tag)
  * [ worktree ](https://git-scm.com/docs/git-worktree)


### Sharing and Updating Projects
  * [ fetch ](https://git-scm.com/docs/git-fetch)
  * [ pull ](https://git-scm.com/docs/git-pull)
  * [ push ](https://git-scm.com/docs/git-push)
  * [ remote ](https://git-scm.com/docs/git-remote)
  * [ submodule ](https://git-scm.com/docs/git-submodule)


### Inspection and Comparison
  * [ show ](https://git-scm.com/docs/git-show)
  * [ log ](https://git-scm.com/docs/git-log)
  * [ diff ](https://git-scm.com/docs/git-diff)
  * [ difftool ](https://git-scm.com/docs/git-difftool)
  * [ range-diff ](https://git-scm.com/docs/git-range-diff)
  * [ shortlog ](https://git-scm.com/docs/git-shortlog)
  * [ describe ](https://git-scm.com/docs/git-describe)


### Patching
  * [ apply ](https://git-scm.com/docs/git-apply)
  * [ cherry-pick ](https://git-scm.com/docs/git-cherry-pick)
  * [ diff ](https://git-scm.com/docs/git-diff)
  * [ rebase ](https://git-scm.com/docs/git-rebase)
  * [ revert ](https://git-scm.com/docs/git-revert)


### Debugging
  * [ bisect ](https://git-scm.com/docs/git-bisect)
  * [ blame ](https://git-scm.com/docs/git-blame)
  * [ grep ](https://git-scm.com/docs/git-grep)


### Email
  * [ am ](https://git-scm.com/docs/git-am)
  * [ apply ](https://git-scm.com/docs/git-apply)
  * [ imap-send ](https://git-scm.com/docs/git-imap-send)
  * [ format-patch ](https://git-scm.com/docs/git-format-patch)
  * [ send-email ](https://git-scm.com/docs/git-send-email)
  * [ request-pull ](https://git-scm.com/docs/git-request-pull)


### External Systems
  * [ svn ](https://git-scm.com/docs/git-svn)
  * [ fast-import ](https://git-scm.com/docs/git-fast-import)


### Server Admin
  * [ daemon ](https://git-scm.com/docs/git-daemon)
  * [ update-server-info ](https://git-scm.com/docs/git-update-server-info)


### Guides
  * [ gitattributes ](https://git-scm.com/docs/gitattributes)
  * [ Command-line interface conventions ](https://git-scm.com/docs/gitcli)
  * [ Everyday Git ](https://git-scm.com/docs/giteveryday)
  * [ Frequently Asked Questions (FAQ) ](https://git-scm.com/docs/gitfaq)
  * [ Glossary ](https://git-scm.com/docs/gitglossary)
  * [ Hooks ](https://git-scm.com/docs/githooks)
  * [ gitignore ](https://git-scm.com/docs/gitignore)
  * [ gitmodules ](https://git-scm.com/docs/gitmodules)
  * [ Revisions ](https://git-scm.com/docs/gitrevisions)
  * [ Submodules ](https://git-scm.com/docs/gitsubmodules)
  * [ Tutorial ](https://git-scm.com/docs/gittutorial)
  * [ Workflows ](https://git-scm.com/docs/gitworkflows)
  * [ All guides... ](https://git-scm.com/docs/git#_guides)


### Administration
  * [ clean ](https://git-scm.com/docs/git-clean)
  * [ gc ](https://git-scm.com/docs/git-gc)
  * [ fsck ](https://git-scm.com/docs/git-fsck)
  * [ reflog ](https://git-scm.com/docs/git-reflog)
  * [ filter-branch ](https://git-scm.com/docs/git-filter-branch)
  * [ instaweb ](https://git-scm.com/docs/git-instaweb)
  * [ archive ](https://git-scm.com/docs/git-archive)
  * [ bundle ](https://git-scm.com/docs/git-bundle)


### Plumbing Commands
  * [ cat-file ](https://git-scm.com/docs/git-cat-file)
  * [ check-ignore ](https://git-scm.com/docs/git-check-ignore)
  * [ checkout-index ](https://git-scm.com/docs/git-checkout-index)
  * [ commit-tree ](https://git-scm.com/docs/git-commit-tree)
  * [ count-objects ](https://git-scm.com/docs/git-count-objects)
  * [ diff-index ](https://git-scm.com/docs/git-diff-index)
  * [ for-each-ref ](https://git-scm.com/docs/git-for-each-ref)
  * [ hash-object ](https://git-scm.com/docs/git-hash-object)
  * [ ls-files ](https://git-scm.com/docs/git-ls-files)
  * [ ls-tree ](https://git-scm.com/docs/git-ls-tree)
  * [ merge-base ](https://git-scm.com/docs/git-merge-base)
  * [ read-tree ](https://git-scm.com/docs/git-read-tree)
  * [ rev-list ](https://git-scm.com/docs/git-rev-list)
  * [ rev-parse ](https://git-scm.com/docs/git-rev-parse)
  * [ show-ref ](https://git-scm.com/docs/git-show-ref)
  * [ symbolic-ref ](https://git-scm.com/docs/git-symbolic-ref)
  * [ update-index ](https://git-scm.com/docs/git-update-index)
  * [ update-ref ](https://git-scm.com/docs/git-update-ref)
  * [ verify-pack ](https://git-scm.com/docs/git-verify-pack)
  * [ write-tree ](https://git-scm.com/docs/git-write-tree)


[ Latest version ▾ ](https://git-scm.com/docs/gitremote-helpers) gitremote-helpers last updated in 2.51.0
Changes in the **gitremote-helpers** manual
  1. 2.51.1 → 2.53.0 no changes
  2. [2.51.0 ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) _2025-08-18_ ](https://git-scm.com/docs/gitremote-helpers/2.51.0)
  3. 2.45.1 → 2.50.1 no changes
  4. [2.45.0 ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) _2024-04-29_ ](https://git-scm.com/docs/gitremote-helpers/2.45.0)
  5. 2.38.1 → 2.44.4 no changes
  6. [2.38.0 ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) _2022-10-02_ ](https://git-scm.com/docs/gitremote-helpers/2.38.0)
  7. 2.28.1 → 2.37.7 no changes
  8. [2.28.0 ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) _2020-07-27_ ](https://git-scm.com/docs/gitremote-helpers/2.28.0)
  9. 2.24.1 → 2.27.1 no changes
  10. [2.24.0 ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) _2019-11-04_ ](https://git-scm.com/docs/gitremote-helpers/2.24.0)
  11. 2.22.1 → 2.23.4 no changes
  12. [2.22.0 ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) _2019-06-07_ ](https://git-scm.com/docs/gitremote-helpers/2.22.0)
  13. 2.18.1 → 2.21.4 no changes
  14. [2.18.0 ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) _2018-06-21_ ](https://git-scm.com/docs/gitremote-helpers/2.18.0)
  15. 2.17.1 → 2.17.6 no changes
  16. [2.17.0 ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) _2018-04-02_ ](https://git-scm.com/docs/gitremote-helpers/2.17.0)
  17. 2.14.6 → 2.16.6 no changes
  18. [2.13.7 ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) _2018-05-22_ ](https://git-scm.com/docs/gitremote-helpers/2.13.7)
  19. [2.12.5 ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) _2017-09-22_ ](https://git-scm.com/docs/gitremote-helpers/2.12.5)
  20. [2.11.4 ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) _2017-09-22_ ](https://git-scm.com/docs/gitremote-helpers/2.11.4)
  21. 2.10.5 no changes
  22. [2.9.5 ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) _2017-07-30_ ](https://git-scm.com/docs/gitremote-helpers/2.9.5)
  23. 2.7.6 → 2.8.6 no changes
  24. [2.6.7 ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) _2017-05-05_ ](https://git-scm.com/docs/gitremote-helpers/2.6.7)
  25. 2.5.6 no changes
  26. [2.4.12 ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) _2017-05-05_ ](https://git-scm.com/docs/gitremote-helpers/2.4.12)
  27. 2.3.10 no changes
  28. [2.2.3 ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) _2015-09-04_ ](https://git-scm.com/docs/gitremote-helpers/2.2.3)
  29. 2.1.4 no changes
  30. [2.0.5 ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/green-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/red-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) ![](https://git-scm.com/images/icons/grey-dot.png) _2014-12-17_ ](https://git-scm.com/docs/gitremote-helpers/2.0.5)


Check your version of git by running
`git --version`
##  [](https://git-scm.com/docs/gitremote-helpers#_name)NAME
gitremote-helpers - Helper programs to interact with remote repositories
##  [](https://git-scm.com/docs/gitremote-helpers#_synopsis)SYNOPSIS
```
_git remote-<transport>_ <repository> [<URL>]
```

##  [](https://git-scm.com/docs/gitremote-helpers#_description)DESCRIPTION
Remote helper programs are normally not used directly by end users, but they are invoked by Git when it needs to interact with remote repositories Git does not support natively. A given helper will implement a subset of the capabilities documented here. When Git needs to interact with a repository using a remote helper, it spawns the helper as an independent process, sends commands to the helper’s standard input, and expects results from the helper’s standard output. Because a remote helper runs as an independent process from Git, there is no need to re-link Git to add a new helper, nor any need to link the helper with the implementation of Git.
Every helper must support the "capabilities" command, which Git uses to determine what other commands the helper will accept. Those other commands can be used to discover and update remote refs, transport objects between the object database and the remote repository, and update the local object store.
Git comes with a "curl" family of remote helpers, that handle various transport protocols, such as _git-remote-http_ , _git-remote-https_ , _git-remote-ftp_ and _git-remote-ftps_. They implement the capabilities _fetch_ , _option_ , and _push_.
##  [](https://git-scm.com/docs/gitremote-helpers#_invocation)INVOCATION
Remote helper programs are invoked with one or (optionally) two arguments. The first argument specifies a remote repository as in Git; it is either the name of a configured remote or a URL. The second argument specifies a URL; it is usually of the form _< transport>://<address>_, but any arbitrary string is possible. The `GIT_DIR` environment variable is set up for the remote helper and can be used to determine where to store additional data or from which directory to invoke auxiliary Git commands.
When Git encounters a URL of the form _< transport>://<address>_, where _< transport>_ is a protocol that it cannot handle natively, it automatically invokes _git remote- <transport>_ with the full URL as the second argument. If such a URL is encountered directly on the command line, the first argument is the same as the second, and if it is encountered in a configured remote, the first argument is the name of that remote.
A URL of the form _< transport>::<address>_ explicitly instructs Git to invoke _git remote- <transport>_ with _< address>_ as the second argument. If such a URL is encountered directly on the command line, the first argument is _< address>_, and if it is encountered in a configured remote, the first argument is the name of that remote.
Additionally, when a configured remote has `remote.`_< name>_`.vcs` set to _< transport>_, Git explicitly invokes _git remote- <transport>_ with _< name>_ as the first argument. If set, the second argument is `remote.`_< name>_`.url`; otherwise, the second argument is omitted.
##  [](https://git-scm.com/docs/gitremote-helpers#_input_format)INPUT FORMAT
Git sends the remote helper a list of commands on standard input, one per line. The first command is always the _capabilities_ command, in response to which the remote helper must print a list of the capabilities it supports (see below) followed by a blank line. The response to the capabilities command determines what commands Git uses in the remainder of the command stream.
The command stream is terminated by a blank line. In some cases (indicated in the documentation of the relevant commands), this blank line is followed by a payload in some other protocol (e.g., the pack protocol), while in others it indicates the end of input.
###  [](https://git-scm.com/docs/gitremote-helpers#_capabilities)Capabilities
Each remote helper is expected to support only a subset of commands. The operations a helper supports are declared to Git in the response to the `capabilities` command (see COMMANDS, below).
In the following, we list all defined capabilities and for each we list which commands a helper with that capability must provide.
####  [](https://git-scm.com/docs/gitremote-helpers#_capabilities_for_pushing)Capabilities for Pushing

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-connect)_connect_

Can attempt to connect to _git receive-pack_ (for pushing), _git upload-pack_ , etc for communication using git’s native packfile protocol. This requires a bidirectional, full-duplex connection.
Supported commands: _connect_.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-stateless-connect)_stateless-connect_

Experimental; for internal use only. Can attempt to connect to a remote server for communication using git’s wire-protocol version 2. See the documentation for the stateless-connect command for more information.
Supported commands: _stateless-connect_.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-push)_push_

Can discover remote refs and push local commits and the history leading up to them to new or existing remote refs.
Supported commands: _list for-push_ , _push_.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-export)_export_

Can discover remote refs and push specified objects from a fast-import stream to remote refs.
Supported commands: _list for-push_ , _export_.
If a helper advertises _connect_ , Git will use it if possible and fall back to another capability if the helper requests so when connecting (see the _connect_ command under COMMANDS). When choosing between _push_ and _export_ , Git prefers _push_. Other frontends may have some other order of preference.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-no-private-update)_no-private-update_

When using the _refspec_ capability, git normally updates the private ref on successful push. This update is disabled when the remote-helper declares the capability _no-private-update_.
####  [](https://git-scm.com/docs/gitremote-helpers#_capabilities_for_fetching)Capabilities for Fetching

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-connect-1)_connect_

Can try to connect to _git upload-pack_ (for fetching), _git receive-pack_ , etc for communication using the Git’s native packfile protocol. This requires a bidirectional, full-duplex connection.
Supported commands: _connect_.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-stateless-connect-1)_stateless-connect_

Experimental; for internal use only. Can attempt to connect to a remote server for communication using git’s wire-protocol version 2. See the documentation for the stateless-connect command for more information.
Supported commands: _stateless-connect_.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-fetch)_fetch_

Can discover remote refs and transfer objects reachable from them to the local object store.
Supported commands: _list_ , _fetch_.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-import)_import_

Can discover remote refs and output objects reachable from them as a stream in fast-import format.
Supported commands: _list_ , _import_.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-check-connectivity)_check-connectivity_

Can guarantee that when a clone is requested, the received pack is self contained and is connected.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-get)_get_

Can use the _get_ command to download a file from a given URI.
If a helper advertises _connect_ , Git will use it if possible and fall back to another capability if the helper requests so when connecting (see the _connect_ command under COMMANDS). When choosing between _fetch_ and _import_ , Git prefers _fetch_. Other frontends may have some other order of preference.
####  [](https://git-scm.com/docs/gitremote-helpers#_miscellaneous_capabilities)Miscellaneous capabilities

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-option)_option_

For specifying settings like `verbosity` (how much output to write to stderr) and `depth` (how much history is wanted in the case of a shallow clone) that affect how other commands are carried out.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-refspecrefspec)_refspec_ <refspec>

For remote helpers that implement _import_ or _export_ , this capability allows the refs to be constrained to a private namespace, instead of writing to refs/heads or refs/remotes directly. It is recommended that all importers providing the _import_ capability use this. It’s mandatory for _export_.
A helper advertising the capability `refspec` `refs/heads/*:refs/svn/origin/branches/*` is saying that, when it is asked to `import` `refs/heads/topic`, the stream it outputs will update the `refs/svn/origin/branches/topic` ref.
This capability can be advertised multiple times. The first applicable refspec takes precedence. The left-hand of refspecs advertised with this capability must cover all refs reported by the list command. If no _refspec_ capability is advertised, there is an implied `refspec` `*:*`.
When writing remote-helpers for decentralized version control systems, it is advised to keep a local copy of the repository to interact with, and to let the private namespace refs point to this local repository, while the refs/remotes namespace is used to track the remote repository.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-bidi-import)_bidi-import_

This modifies the _import_ capability. The fast-import commands _cat-blob_ and _ls_ can be used by remote-helpers to retrieve information about blobs and trees that already exist in fast-import’s memory. This requires a channel from fast-import to the remote-helper. If it is advertised in addition to "import", Git establishes a pipe from fast-import to the remote-helper’s stdin. It follows that Git and fast-import are both connected to the remote-helper’s stdin. Because Git can send multiple commands to the remote-helper it is required that helpers that use _bidi-import_ buffer all _import_ commands of a batch before sending data to fast-import. This is to prevent mixing commands and fast-import responses on the helper’s stdin.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-export-marksfile)_export-marks_ <file>

This modifies the _export_ capability, instructing Git to dump the internal marks table to <file> when complete. For details, read up on `--export-marks=`_< file>_ in [git-fast-export[1]](https://git-scm.com/docs/git-fast-export).

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-import-marksfile)_import-marks_ <file>

This modifies the _export_ capability, instructing Git to load the marks specified in <file> before processing any input. For details, read up on `--import-marks=`_< file>_ in [git-fast-export[1]](https://git-scm.com/docs/git-fast-export).

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-signed-tags)_signed-tags_

This modifies the _export_ capability, instructing Git to pass `--signed-tags=verbatim` to [git-fast-export[1]](https://git-scm.com/docs/git-fast-export). In the absence of this capability, Git will use `--signed-tags=warn-strip`.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-object-format)_object-format_

This indicates that the helper is able to interact with the remote side using an explicit hash algorithm extension.
##  [](https://git-scm.com/docs/gitremote-helpers#_commands)COMMANDS
Commands are given by the caller on the helper’s standard input, one per line.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-capabilities)_capabilities_

Lists the capabilities of the helper, one per line, ending with a blank line. Each capability may be preceded with _*_ , which marks them mandatory for Git versions using the remote helper to understand. Any unknown mandatory capability is a fatal error.
Support for this command is mandatory.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-list)_list_

Lists the refs, one per line, in the format "<value> <name> [<attr> …​]". The value may be a hex sha1 hash, "@<dest>" for a symref, ":<keyword> <value>" for a key-value pair, or "?" to indicate that the helper could not get the value of the ref. A space-separated list of attributes follows the name; unrecognized attributes are ignored. The list ends with a blank line.
See REF LIST ATTRIBUTES for a list of currently defined attributes. See REF LIST KEYWORDS for a list of currently defined keywords.
Supported if the helper has the "fetch" or "import" capability.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-listfor-push)_list for-push_

Similar to _list_ , except that it is used if and only if the caller wants to the resulting ref list to prepare push commands. A helper supporting both push and fetch can use this to distinguish for which operation the output of _list_ is going to be used, possibly reducing the amount of work that needs to be performed.
Supported if the helper has the "push" or "export" capability.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-optionnamevalue)_option_ <name> <value>

Sets the transport helper option <name> to <value>. Outputs a single line containing one of _ok_ (option successfully set), _unsupported_ (option not recognized) or _error <msg>_ (option <name> is supported but <value> is not valid for it). Options should be set before other commands, and may influence the behavior of those commands.
See OPTIONS for a list of currently defined options.
Supported if the helper has the "option" capability.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-fetchsha1name)_fetch_ <sha1> <name>

Fetches the given object, writing the necessary objects to the database. Fetch commands are sent in a batch, one per line, terminated with a blank line. Outputs a single blank line when all fetch commands in the same batch are complete. Only objects which were reported in the output of _list_ with a sha1 may be fetched this way.
Optionally may output a _lock <file>_ line indicating the full path of a file under `$GIT_DIR/objects/pack` which is keeping a pack until refs can be suitably updated. The path must end with `.keep`. This is a mechanism to name a <pack,idx,keep> tuple by giving only the keep component. The kept pack will not be deleted by a concurrent repack, even though its objects may not be referenced until the fetch completes. The `.keep` file will be deleted at the conclusion of the fetch.
If option _check-connectivity_ is requested, the helper must output _connectivity-ok_ if the clone is self-contained and connected.
Supported if the helper has the "fetch" capability.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-pushsrcdst)_push_ +<src>:<dst>

Pushes the given local <src> commit or branch to the remote branch described by <dst>. A batch sequence of one or more _push_ commands is terminated with a blank line (if there is only one reference to push, a single _push_ command is followed by a blank line). For example, the following would be two batches of _push_ , the first asking the remote-helper to push the local ref _master_ to the remote ref _master_ and the local `HEAD` to the remote _branch_ , and the second asking to push ref _foo_ to ref _bar_ (forced update requested by the _+_).
```
push refs/heads/master:refs/heads/master
push HEAD:refs/heads/branch
\n
push +refs/heads/foo:refs/heads/bar
\n
```

Zero or more protocol options may be entered after the last _push_ command, before the batch’s terminating blank line.
When the push is complete, outputs one or more _ok <dst>_ or _error <dst> <why>?_ lines to indicate success or failure of each pushed ref. The status report output is terminated by a blank line. The option field <why> may be quoted in a C style string if it contains an LF.
Supported if the helper has the "push" capability.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-importname)_import_ <name>

Produces a fast-import stream which imports the current value of the named ref. It may additionally import other refs as needed to construct the history efficiently. The script writes to a helper-specific private namespace. The value of the named ref should be written to a location in this namespace derived by applying the refspecs from the "refspec" capability to the name of the ref.
Especially useful for interoperability with a foreign versioning system.
Just like _push_ , a batch sequence of one or more _import_ is terminated with a blank line. For each batch of _import_ , the remote helper should produce a fast-import stream terminated by a _done_ command.
Note that if the _bidi-import_ capability is used the complete batch sequence has to be buffered before starting to send data to fast-import to prevent mixing of commands and fast-import responses on the helper’s stdin.
Supported if the helper has the "import" capability.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-export-1)_export_

Instructs the remote helper that any subsequent input is part of a fast-import stream (generated by _git fast-export_) containing objects which should be pushed to the remote.
Especially useful for interoperability with a foreign versioning system.
The _export-marks_ and _import-marks_ capabilities, if specified, affect this command in so far as they are passed on to _git fast-export_ , which then will load/store a table of marks for local objects. This can be used to implement for incremental operations.
Supported if the helper has the "export" capability.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-connectservice)_connect_ <service>

Connects to given service. Standard input and standard output of helper are connected to specified service (git prefix is included in service name so e.g. fetching uses _git-upload-pack_ as service) on remote side. Valid replies to this command are empty line (connection established), _fallback_ (no smart transport support, fall back to dumb transports) and just exiting with error message printed (can’t connect, don’t bother trying to fall back). After line feed terminating the positive (empty) response, the output of service starts. After the connection ends, the remote helper exits.
Supported if the helper has the "connect" capability.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-stateless-connectservice)_stateless-connect_ <service>

Experimental; for internal use only. Connects to the given remote service for communication using git’s wire-protocol version 2. Valid replies to this command are empty line (connection established), _fallback_ (no smart transport support, fall back to dumb transports) and just exiting with error message printed (can’t connect, don’t bother trying to fall back). After line feed terminating the positive (empty) response, the output of the service starts. Messages (both request and response) must consist of zero or more PKT-LINEs, terminating in a flush packet. Response messages will then have a response end packet after the flush packet to indicate the end of a response. The client must not expect the server to store any state in between request-response pairs. After the connection ends, the remote helper exits.
Supported if the helper has the "stateless-connect" capability.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-geturipath)_get_ <uri> <path>

Downloads the file from the given _< uri>_ to the given _< path>_. If _< path>_`.temp` exists, then Git assumes that the `.temp` file is a partial download from a previous attempt and will resume the download from that position.
If a fatal error occurs, the program writes the error message to stderr and exits. The caller should expect that a suitable error message has been printed if the child closes the connection without completing a valid response for the current command.
Additional commands may be supported, as may be determined from capabilities reported by the helper.
##  [](https://git-scm.com/docs/gitremote-helpers#_ref_list_attributes)REF LIST ATTRIBUTES
The _list_ command produces a list of refs in which each ref may be followed by a list of attributes. The following ref list attributes are defined.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-unchanged)_unchanged_

This ref is unchanged since the last import or fetch, although the helper cannot necessarily determine what value that produced.
##  [](https://git-scm.com/docs/gitremote-helpers#_ref_list_keywords)REF LIST KEYWORDS
The _list_ command may produce a list of key-value pairs. The following keys are defined.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-object-format-1)_object-format_

The refs are using the given hash algorithm. This keyword is only used if the server and client both support the object-format extension.
##  [](https://git-scm.com/docs/gitremote-helpers#_options)OPTIONS
The following options are defined and (under suitable circumstances) set by Git if the remote helper has the _option_ capability.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-optionverbosityn)_option verbosity_ <n>

Changes the verbosity of messages displayed by the helper. A value of 0 for <n> means that processes operate quietly, and the helper produces only error output. 1 is the default level of verbosity, and higher values of <n> correspond to the number of -v flags passed on the command line.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-optionprogresstruefalse)_option progress_ {_true_ |_false_}

Enables (or disables) progress messages displayed by the transport helper during a command.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-optiondepthdepth)_option depth_ <depth>

Deepens the history of a shallow repository.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-optiondeepen-sincetimestamp)_option deepen-since_ <timestamp>

Deepens the history of a shallow repository based on time.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-optiondeepen-notref)_option deepen-not_ <ref>

Deepens the history of a shallow repository excluding ref. Multiple options add up.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-optiondeepen-relativetruefalse)_option deepen-relative_ {_true_ |_false_}

Deepens the history of a shallow repository relative to current boundary. Only valid when used with "option depth".

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-optionfollowtagstruefalse)_option followtags_ {_true_ |_false_}

If enabled the helper should automatically fetch annotated tag objects if the object the tag points at was transferred during the fetch command. If the tag is not fetched by the helper a second fetch command will usually be sent to ask for the tag specifically. Some helpers may be able to use this option to avoid a second network connection.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-optiondry-runtruefalse)_option dry-run_ {_true_ |_false_}

If true, pretend the operation completed successfully, but don’t actually change any repository data. For most helpers this only applies to the _push_ , if supported.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-optionservpathc-style-quoted-path)_option servpath <c-style-quoted-path>_

Sets service path (--upload-pack, --receive-pack etc.) for next connect. Remote helper may support this option, but must not rely on this option being set before connect request occurs.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-optioncheck-connectivitytruefalse)_option check-connectivity_ {_true_ |_false_}

Request the helper to check connectivity of a clone.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-optionforcetruefalse)_option force_ {_true_ |_false_}

Request the helper to perform a force update. Defaults to _false_.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-optioncloningtruefalse)_option cloning_ {_true_ |_false_}

Notify the helper this is a clone request (i.e. the current repository is guaranteed empty).

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-optionupdate-shallowtruefalse)_option update-shallow_ {_true_ |_false_}

Allow to extend .git/shallow if the new refs require it.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-optionpushcerttruefalse)_option pushcert_ {_true_ |_false_}

GPG sign pushes.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-optionpush-optionstring)_option push-option_ <string>

Transmit <string> as a push option. As the push option must not contain LF or NUL characters, the string is not encoded.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-optionfrom-promisortruefalse)_option from-promisor_ {_true_ |_false_}

Indicate that these objects are being fetched from a promisor.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-optionno-dependentstruefalse)_option no-dependents_ {_true_ |_false_}

Indicate that only the objects wanted need to be fetched, not their dependents.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-optionatomictruefalse)_option atomic_ {_true_ |_false_}

When pushing, request the remote server to update refs in a single atomic transaction. If successful, all refs will be updated, or none will. If the remote side does not support this capability, the push will fail.

[](https://git-scm.com/docs/gitremote-helpers#Documentation/gitremote-helpers.txt-optionobject-formattrue)_option object-format true_

Indicate that the caller wants hash algorithm information to be passed back from the remote. This mode is used when fetching refs.
##  [](https://git-scm.com/docs/gitremote-helpers#_see_also)SEE ALSO
[git-remote[1]](https://git-scm.com/docs/git-remote)
[git-remote-ext[1]](https://git-scm.com/docs/git-remote-ext)
[git-remote-fd[1]](https://git-scm.com/docs/git-remote-fd)
[git-fast-import[1]](https://git-scm.com/docs/git-fast-import)
##  [](https://git-scm.com/docs/gitremote-helpers#_git)GIT
Part of the [git[1]](https://git-scm.com/docs/git) suite
### gitremote-helpers
[About this site](https://git-scm.com/site)
Patches, suggestions, and comments are welcome.
Git is a member of [Software Freedom Conservancy](https://git-scm.com/sfc)
